@@ -65,11 +65,18 @@ impl IProfCollectd for ProfcollectdBinderService {
             .context("Failed to terminate collection.")
             .map_err(err_to_binder_status)
     }
-    fn trace_once(&self, tag: &str) -> BinderResult<()> {
+    fn trace_system(&self, tag: &str) -> BinderResult<()> {
         let lock = &mut *self.lock();
         lock.scheduler
-            .one_shot(&lock.config, tag)
-            .context("Failed to initiate an one-off trace.")
+            .trace_system(&lock.config, tag)
+            .context("Failed to perform system-wide trace.")
+            .map_err(err_to_binder_status)
+    }
+    fn trace_process(&self, tag: &str, process: &str, duration: f32) -> BinderResult<()> {
+        let lock = &mut *self.lock();
+        lock.scheduler
+            .trace_process(&lock.config, tag, process, duration)
+            .context("Failed to perform process trace.")
             .map_err(err_to_binder_status)
     }
     fn process(&self) -> BinderResult<()> {
