@@ -11,7 +11,7 @@
 #include "../../../Windows/ErrorMsg.h"
 #include "../../../Windows/PropVariantConv.h"
 
-#ifndef _7ZIP_ST
+#ifndef Z7_ST
 #include "../../../Windows/Synchronization.h"
 #endif
 
@@ -56,19 +56,27 @@ HRESULT CExtractScanConsole::ScanProgress(const CDirItemsStat &st, const FString
 
 HRESULT CExtractScanConsole::ScanError(const FString &path, DWORD systemError)
 {
+  // 22.00:
+  // ScanErrors.AddError(path, systemError);
+
   ClosePercentsAndFlush();
   
   if (_se)
   {
     *_se << endl << kError << NError::MyFormatMessage(systemError) << endl;
-    _se->NormalizePrint_UString(fs2us(path));
+    _se->NormalizePrint_UString_Path(fs2us(path));
     *_se << endl << endl;
     _se->Flush();
   }
   return HRESULT_FROM_WIN32(systemError);
+
+  // 22.00: commented
+  // CommonError(path, systemError, true);
+  // return S_OK;
 }
 
 
+void Print_UInt64_and_String(AString &s, UInt64 val, const char *name);
 void Print_UInt64_and_String(AString &s, UInt64 val, const char *name)
 {
   char temp[32];
@@ -78,6 +86,7 @@ void Print_UInt64_and_String(AString &s, UInt64 val, const char *name)
   s += name;
 }
 
+void PrintSize_bytes_Smart(AString &s, UInt64 val);
 void PrintSize_bytes_Smart(AString &s, UInt64 val)
 {
   Print_UInt64_and_String(s, val, "bytes");
@@ -93,10 +102,10 @@ void PrintSize_bytes_Smart(AString &s, UInt64 val)
   temp[0] = c;
   s += " (";
   Print_UInt64_and_String(s, ((val + ((UInt64)1 << numBits) - 1) >> numBits), temp);
-  s += ')';
+  s.Add_Char(')');
 }
 
-void PrintSize_bytes_Smart_comma(AString &s, UInt64 val)
+static void PrintSize_bytes_Smart_comma(AString &s, UInt64 val)
 {
   if (val == (UInt64)(Int64)-1)
     return;
@@ -106,6 +115,7 @@ void PrintSize_bytes_Smart_comma(AString &s, UInt64 val)
 
 
 
+void Print_DirItemsStat(AString &s, const CDirItemsStat &st);
 void Print_DirItemsStat(AString &s, const CDirItemsStat &st)
 {
   if (st.NumDirs != 0)
@@ -124,6 +134,7 @@ void Print_DirItemsStat(AString &s, const CDirItemsStat &st)
 }
 
 
+void Print_DirItemsStat2(AString &s, const CDirItemsStat2 &st);
 void Print_DirItemsStat2(AString &s, const CDirItemsStat2 &st)
 {
   Print_DirItemsStat(s, (CDirItemsStat &)st);
@@ -172,7 +183,7 @@ void CExtractScanConsole::PrintStat(const CDirItemsStat &st)
 
 
 
-#ifndef _7ZIP_ST
+#ifndef Z7_ST
 static NSynchronization::CCriticalSection g_CriticalSection;
 #define MT_LOCK NSynchronization::CCriticalSectionLock lock(g_CriticalSection);
 #else
@@ -183,10 +194,11 @@ static NSynchronization::CCriticalSection g_CriticalSection;
 static const char * const kTestString    =  "T";
 static const char * const kExtractString =  "-";
 static const char * const kSkipString    =  ".";
+static const char * const kReadString    =  "H";
 
-// static const char * const kCantAutoRename = "can not create file with auto name\n";
-// static const char * const kCantRenameFile = "can not rename existing file\n";
-// static const char * const kCantDeleteOutputFile = "can not delete output file ";
+// static const char * const kCantAutoRename = "cannot create file with auto name\n";
+// static const char * const kCantRenameFile = "cannot rename existing file\n";
+// static const char * const kCantDeleteOutputFile = "cannot delete output file ";
 
 static const char * const kMemoryExceptionMessage = "Can't allocate required memory!";
 
@@ -223,7 +235,7 @@ static const char * const k_ErrorFlagsMessages[] =
   , "CRC Error"
 };
 
-STDMETHODIMP CExtractCallbackConsole::SetTotal(UInt64 size)
+Z7_COM7F_IMF(CExtractCallbackConsole::SetTotal(UInt64 size))
 {
   MT_LOCK
 
@@ -235,7 +247,7 @@ STDMETHODIMP CExtractCallbackConsole::SetTotal(UInt64 size)
   return CheckBreak2();
 }
 
-STDMETHODIMP CExtractCallbackConsole::SetCompleted(const UInt64 *completeValue)
+Z7_COM7F_IMF(CExtractCallbackConsole::SetCompleted(const UInt64 *completeValue))
 {
   MT_LOCK
 
@@ -253,7 +265,7 @@ static const char * const kTab = "  ";
 static void PrintFileInfo(CStdOutStream *_so, const wchar_t *path, const FILETIME *ft, const UInt64 *size)
 {
   *_so << kTab << "Path:     ";
-  _so->NormalizePrint_wstr(path);
+  _so->NormalizePrint_wstr_Path(path);
   *_so << endl;
   if (size && *size != (UInt64)(Int64)-1)
   {
@@ -269,14 +281,14 @@ static void PrintFileInfo(CStdOutStream *_so, const wchar_t *path, const FILETIM
   }
 }
 
-STDMETHODIMP CExtractCallbackConsole::AskOverwrite(
+Z7_COM7F_IMF(CExtractCallbackConsole::AskOverwrite(
     const wchar_t *existName, const FILETIME *existTime, const UInt64 *existSize,
     const wchar_t *newName, const FILETIME *newTime, const UInt64 *newSize,
-    Int32 *answer)
+    Int32 *answer))
 {
   MT_LOCK
   
-  RINOK(CheckBreak2());
+  RINOK(CheckBreak2())
 
   ClosePercentsAndFlush();
 
@@ -290,7 +302,7 @@ STDMETHODIMP CExtractCallbackConsole::AskOverwrite(
   
   NUserAnswerMode::EEnum overwriteAnswer = ScanUserYesNoAllQuit(_so);
   
-  switch (overwriteAnswer)
+  switch ((int)overwriteAnswer)
   {
     case NUserAnswerMode::kQuit:  return E_ABORT;
     case NUserAnswerMode::kNo:     *answer = NOverwriteAnswer::kNo; break;
@@ -313,7 +325,7 @@ STDMETHODIMP CExtractCallbackConsole::AskOverwrite(
   return CheckBreak2();
 }
 
-STDMETHODIMP CExtractCallbackConsole::PrepareOperation(const wchar_t *name, Int32 /* isFolder */, Int32 askExtractMode, const UInt64 *position)
+Z7_COM7F_IMF(CExtractCallbackConsole::PrepareOperation(const wchar_t *name, Int32 isFolder, Int32 askExtractMode, const UInt64 *position))
 {
   MT_LOCK
   
@@ -327,8 +339,9 @@ STDMETHODIMP CExtractCallbackConsole::PrepareOperation(const wchar_t *name, Int3
     case NArchive::NExtract::NAskMode::kExtract: s = kExtractString; break;
     case NArchive::NExtract::NAskMode::kTest:    s = kTestString; break;
     case NArchive::NExtract::NAskMode::kSkip:    s = kSkipString; requiredLevel = 2; break;
+    case NArchive::NExtract::NAskMode::kReadExternal: s = kReadString; requiredLevel = 0; break;
     default: s = "???"; requiredLevel = 2;
-  };
+  }
 
   bool show2 = (LogLevel >= requiredLevel && _so);
 
@@ -345,7 +358,13 @@ STDMETHODIMP CExtractCallbackConsole::PrepareOperation(const wchar_t *name, Int3
     if (name)
     {
       _tempU = name;
-      _so->Normalize_UString(_tempU);
+      _so->Normalize_UString_Path(_tempU);
+      // 21.04
+      if (isFolder)
+      {
+        if (!_tempU.IsEmpty() && _tempU.Back() != WCHAR_PATH_SEPARATOR)
+          _tempU.Add_PathSepar();
+      }
     }
     _so->PrintUString(_tempU, _tempA);
     if (position)
@@ -375,11 +394,11 @@ STDMETHODIMP CExtractCallbackConsole::PrepareOperation(const wchar_t *name, Int3
   return CheckBreak2();
 }
 
-STDMETHODIMP CExtractCallbackConsole::MessageError(const wchar_t *message)
+Z7_COM7F_IMF(CExtractCallbackConsole::MessageError(const wchar_t *message))
 {
   MT_LOCK
   
-  RINOK(CheckBreak2());
+  RINOK(CheckBreak2())
 
   NumFileErrors_in_Current++;
   NumFileErrors++;
@@ -394,6 +413,7 @@ STDMETHODIMP CExtractCallbackConsole::MessageError(const wchar_t *message)
   return CheckBreak2();
 }
 
+void SetExtractErrorMessage(Int32 opRes, Int32 encrypted, AString &dest);
 void SetExtractErrorMessage(Int32 opRes, Int32 encrypted, AString &dest)
 {
   dest.Empty();
@@ -428,6 +448,7 @@ void SetExtractErrorMessage(Int32 opRes, Int32 encrypted, AString &dest)
       case NArchive::NExtract::NOperationResult::kWrongPassword:
         s = kWrongPassword;
         break;
+      default: break;
     }
     
     dest += kError;
@@ -436,11 +457,11 @@ void SetExtractErrorMessage(Int32 opRes, Int32 encrypted, AString &dest)
     else
     {
       dest += "Error #";
-      dest.Add_UInt32(opRes);
+      dest.Add_UInt32((UInt32)opRes);
     }
 }
 
-STDMETHODIMP CExtractCallbackConsole::SetOperationResult(Int32 opRes, Int32 encrypted)
+Z7_COM7F_IMF(CExtractCallbackConsole::SetOperationResult(Int32 opRes, Int32 encrypted))
 {
   MT_LOCK
   
@@ -469,7 +490,7 @@ STDMETHODIMP CExtractCallbackConsole::SetOperationResult(Int32 opRes, Int32 encr
       if (!_currentName.IsEmpty())
       {
         *_se << " : ";
-        _se->NormalizePrint_UString(_currentName);
+        _se->NormalizePrint_UString_Path(_currentName);
       }
       *_se << endl;
       _se->Flush();
@@ -479,7 +500,7 @@ STDMETHODIMP CExtractCallbackConsole::SetOperationResult(Int32 opRes, Int32 encr
   return CheckBreak2();
 }
 
-STDMETHODIMP CExtractCallbackConsole::ReportExtractResult(Int32 opRes, Int32 encrypted, const wchar_t *name)
+Z7_COM7F_IMF(CExtractCallbackConsole::ReportExtractResult(Int32 opRes, Int32 encrypted, const wchar_t *name))
 {
   if (opRes != NArchive::NExtract::NOperationResult::kOK)
   {
@@ -492,7 +513,7 @@ STDMETHODIMP CExtractCallbackConsole::ReportExtractResult(Int32 opRes, Int32 enc
 
 
 
-#ifndef _NO_CRYPTO
+#ifndef Z7_NO_CRYPTO
 
 HRESULT CExtractCallbackConsole::SetPassword(const UString &password)
 {
@@ -501,7 +522,7 @@ HRESULT CExtractCallbackConsole::SetPassword(const UString &password)
   return S_OK;
 }
 
-STDMETHODIMP CExtractCallbackConsole::CryptoGetTextPassword(BSTR *password)
+Z7_COM7F_IMF(CExtractCallbackConsole::CryptoGetTextPassword(BSTR *password))
 {
   COM_TRY_BEGIN
   MT_LOCK
@@ -511,9 +532,101 @@ STDMETHODIMP CExtractCallbackConsole::CryptoGetTextPassword(BSTR *password)
 
 #endif
 
+
+#ifndef Z7_SFX
+
+void CExtractCallbackConsole::PrintTo_se_Path_WithTitle(const UString &path, const char *title)
+{
+  *_se << title;
+  _se->NormalizePrint_UString_Path(path);
+  *_se << endl;
+}
+
+void CExtractCallbackConsole::Add_ArchiveName_Error()
+{
+  if (_needWriteArchivePath)
+  {
+    PrintTo_se_Path_WithTitle(_currentArchivePath, "Archive: ");
+    _needWriteArchivePath = false;
+  }
+}
+
+
+Z7_COM7F_IMF(CExtractCallbackConsole::RequestMemoryUse(
+    UInt32 flags, UInt32 /* indexType */, UInt32 /* index */, const wchar_t *path,
+    UInt64 requiredSize, UInt64 *allowedSize, UInt32 *answerFlags))
+{
+  if ((flags & NRequestMemoryUseFlags::k_IsReport) == 0
+      && requiredSize <= *allowedSize)
+  {
+#if 0
+    // it's expected, that *answerFlags was set to NRequestMemoryAnswerFlags::k_Allow already,
+    // because it's default answer for (requiredSize <= *allowedSize) case.
+    // optional code:
+    *answerFlags = NRequestMemoryAnswerFlags::k_Allow;
+#endif
+  }
+  else
+  {
+    if ((flags & NRequestMemoryUseFlags::k_NoErrorMessage) == 0)
+    if (_se)
+    {
+      const UInt64 num_GB_allowed  = (*allowedSize + ((1u << 30) - 1)) >> 30;
+      const UInt64 num_GB_required = (requiredSize + ((1u << 30) - 1)) >> 30;
+      ClosePercentsAndFlush();
+      Add_ArchiveName_Error();
+      if (path)
+        PrintTo_se_Path_WithTitle(path, "File: ");
+      *_se << "The extraction operation requires big amount memory (RAM):" << endl
+        << "  " << num_GB_required  << " GB : required memory usage size" << endl
+        << "  " << num_GB_allowed   << " GB : allowed memory usage limit" << endl
+        << "  Use -smemx{size}g switch to set allowed memory usage limit for extraction." << endl;
+      *_se << "ERROR: Memory usage limit was exceeded." << endl;
+      const char *m = NULL;
+      // if (indexType == NArchive::NEventIndexType::kNoIndex)
+           if ((flags & NRequestMemoryUseFlags::k_SkipArc_IsExpected) ||
+               (flags & NRequestMemoryUseFlags::k_Report_SkipArc))
+          m = "Archive unpacking was skipped.";
+/*
+      else if ((flags & NRequestMemoryUseFlags::k_SkipBigFiles_IsExpected) ||
+               (flags & NRequestMemoryUseFlags::k_Report_SkipBigFiles))
+          m = "Extraction for some files will be skipped.";
+      else if ((flags & NRequestMemoryUseFlags::k_SkipBigFile_IsExpected) ||
+               (flags & NRequestMemoryUseFlags::k_Report_SkipBigFile))
+          m = "File extraction was skipped.";
+*/
+      if (m)
+        *_se << m;
+      _se->Flush();
+    }
+
+    if ((flags & NRequestMemoryUseFlags::k_IsReport) == 0)
+    {
+      // default answer can be k_Allow, if limit was not forced,
+      // so we change answer to non-allowed here.
+      *answerFlags = NRequestMemoryAnswerFlags::k_Limit_Exceeded;
+           if (flags & NRequestMemoryUseFlags::k_SkipArc_IsExpected)
+        *answerFlags |= NRequestMemoryAnswerFlags::k_SkipArc;
+/*
+      else if (flags & NRequestMemoryUseFlags::k_SkipBigFile_IsExpected)
+        *answerFlags |= NRequestMemoryAnswerFlags::k_SkipBigFile;
+      else if (flags & NRequestMemoryUseFlags::k_SkipBigFiles_IsExpected)
+        *answerFlags |= NRequestMemoryAnswerFlags::k_SkipBigFiles;
+*/
+    }
+  }
+  return CheckBreak2();
+}
+
+#endif
+
+
 HRESULT CExtractCallbackConsole::BeforeOpen(const wchar_t *name, bool testMode)
 {
-  RINOK(CheckBreak2());
+  _currentArchivePath = name;
+  _needWriteArchivePath = true;
+
+  RINOK(CheckBreak2())
 
   NumTryArcs++;
   ThereIsError_in_Current = false;
@@ -524,7 +637,7 @@ HRESULT CExtractCallbackConsole::BeforeOpen(const wchar_t *name, bool testMode)
   if (_so)
   {
     *_so << endl << (testMode ? kTesting : kExtracting);
-    _so->NormalizePrint_wstr(name);
+    _so->NormalizePrint_wstr_Path(name);
     *_so << endl;
   }
 
@@ -540,7 +653,7 @@ static AString GetOpenArcErrorMessage(UInt32 errorFlags)
 {
   AString s;
   
-  for (unsigned i = 0; i < ARRAY_SIZE(k_ErrorFlagsMessages); i++)
+  for (unsigned i = 0; i < Z7_ARRAY_SIZE(k_ErrorFlagsMessages); i++)
   {
     UInt32 f = (1 << i);
     if ((errorFlags & f) == 0)
@@ -566,6 +679,7 @@ static AString GetOpenArcErrorMessage(UInt32 errorFlags)
   return s;
 }
 
+void PrintErrorFlags(CStdOutStream &so, const char *s, UInt32 errorFlags);
 void PrintErrorFlags(CStdOutStream &so, const char *s, UInt32 errorFlags)
 {
   if (errorFlags == 0)
@@ -573,7 +687,7 @@ void PrintErrorFlags(CStdOutStream &so, const char *s, UInt32 errorFlags)
   so << s << endl << GetOpenArcErrorMessage(errorFlags) << endl;
 }
 
-void Add_Messsage_Pre_ArcType(UString &s, const char *pre, const wchar_t *arcType)
+static void Add_Messsage_Pre_ArcType(UString &s, const char *pre, const wchar_t *arcType)
 {
   s.Add_LF();
   s += pre;
@@ -582,12 +696,13 @@ void Add_Messsage_Pre_ArcType(UString &s, const char *pre, const wchar_t *arcTyp
   s += "] archive";
 }
 
+void Print_ErrorFormatIndex_Warning(CStdOutStream *_so, const CCodecs *codecs, const CArc &arc);
 void Print_ErrorFormatIndex_Warning(CStdOutStream *_so, const CCodecs *codecs, const CArc &arc)
 {
   const CArcErrorInfo &er = arc.ErrorInfo;
   
   *_so << "WARNING:\n";
-  _so->NormalizePrint_UString(arc.Path);
+  _so->NormalizePrint_UString_Path(arc.Path);
   UString s;
   if (arc.FormatIndex == er.ErrorFormatIndex)
   {
@@ -596,7 +711,7 @@ void Print_ErrorFormatIndex_Warning(CStdOutStream *_so, const CCodecs *codecs, c
   }
   else
   {
-    Add_Messsage_Pre_ArcType(s, "Can not open the file", codecs->GetFormatNamePtr(er.ErrorFormatIndex));
+    Add_Messsage_Pre_ArcType(s, "Cannot open the file", codecs->GetFormatNamePtr(er.ErrorFormatIndex));
     Add_Messsage_Pre_ArcType(s, "The file is open", codecs->GetFormatNamePtr(arc.FormatIndex));
   }
   
@@ -608,6 +723,9 @@ HRESULT CExtractCallbackConsole::OpenResult(
     const CCodecs *codecs, const CArchiveLink &arcLink,
     const wchar_t *name, HRESULT result)
 {
+  _currentArchivePath = name;
+  _needWriteArchivePath = true;
+
   ClosePercents();
 
   if (NeedPercents())
@@ -634,7 +752,7 @@ HRESULT CExtractCallbackConsole::OpenResult(
         *_se << endl;
         if (level != 0)
         {
-          _se->NormalizePrint_UString(arc.Path);
+          _se->NormalizePrint_UString_Path(arc.Path);
           *_se << endl;
         }
       }
@@ -671,7 +789,7 @@ HRESULT CExtractCallbackConsole::OpenResult(
         *_so << endl;
         if (level != 0)
         {
-          _so->NormalizePrint_UString(arc.Path);
+          _so->NormalizePrint_UString_Path(arc.Path);
           *_so << endl;
         }
       }
@@ -717,7 +835,7 @@ HRESULT CExtractCallbackConsole::OpenResult(
   {
     if (_so)
     {
-      RINOK(Print_OpenArchive_Props(*_so, codecs, arcLink));
+      RINOK(Print_OpenArchive_Props(*_so, codecs, arcLink))
       *_so << endl;
     }
   }
@@ -729,10 +847,10 @@ HRESULT CExtractCallbackConsole::OpenResult(
     if (_se)
     {
       *_se << kError;
-      _se->NormalizePrint_wstr(name);
+      _se->NormalizePrint_wstr_Path(name);
       *_se << endl;
-      HRESULT res = Print_OpenArchive_Error(*_se, codecs, arcLink);
-      RINOK(res);
+      const HRESULT res = Print_OpenArchive_Error(*_se, codecs, arcLink);
+      RINOK(res)
       if (result == S_FALSE)
       {
       }
@@ -806,7 +924,9 @@ HRESULT CExtractCallbackConsole::ExtractResult(HRESULT result)
   else
   {
     NumArcsWithError++;
-    if (result == E_ABORT || result == ERROR_DISK_FULL)
+    if (result == E_ABORT
+        || result == HRESULT_FROM_WIN32(ERROR_DISK_FULL)
+        )
       return result;
     
     if (_se)
