@@ -5,9 +5,40 @@
 #include "../../../Common/MyWindows.h"
 
 #ifdef _WIN32
+
+#ifndef Z7_OLD_WIN_SDK
+
+#if defined(__MINGW32__) || defined(__MINGW64__)
+#include <psapi.h>
+#else
 #include <Psapi.h>
 #endif
 
+#else // Z7_OLD_WIN_SDK
+
+typedef struct {
+    DWORD cb;
+    DWORD PageFaultCount;
+    SIZE_T PeakWorkingSetSize;
+    SIZE_T WorkingSetSize;
+    SIZE_T QuotaPeakPagedPoolUsage;
+    SIZE_T QuotaPagedPoolUsage;
+    SIZE_T QuotaPeakNonPagedPoolUsage;
+    SIZE_T QuotaNonPagedPoolUsage;
+    SIZE_T PagefileUsage;
+    SIZE_T PeakPagefileUsage;
+} PROCESS_MEMORY_COUNTERS;
+typedef PROCESS_MEMORY_COUNTERS *PPROCESS_MEMORY_COUNTERS;
+
+#endif // Z7_OLD_WIN_SDK
+
+#else // _WIN32
+#include <unistd.h>
+#include <sys/ioctl.h>
+#include <sys/time.h>
+#include <sys/times.h>
+#endif // _WIN32
+
 #include "../../../../C/CpuArch.h"
 
 #include "../../../Common/MyInitGuid.h"
@@ -15,20 +46,22 @@
 #include "../../../Common/CommandLineParser.h"
 #include "../../../Common/IntToString.h"
 #include "../../../Common/MyException.h"
+#include "../../../Common/StdInStream.h"
+#include "../../../Common/StdOutStream.h"
 #include "../../../Common/StringConvert.h"
 #include "../../../Common/StringToInt.h"
 #include "../../../Common/UTFConvert.h"
 
 #include "../../../Windows/ErrorMsg.h"
-
 #include "../../../Windows/TimeUtils.h"
+#include "../../../Windows/FileDir.h"
 
 #include "../Common/ArchiveCommandLine.h"
 #include "../Common/Bench.h"
 #include "../Common/ExitCode.h"
 #include "../Common/Extract.h"
 
-#ifdef EXTERNAL_CODECS
+#ifdef Z7_EXTERNAL_CODECS
 #include "../Common/LoadCodecs.h"
 #endif
 
@@ -37,13 +70,12 @@
 #include "BenchCon.h"
 #include "ConsoleClose.h"
 #include "ExtractCallbackConsole.h"
+#include "HashCon.h"
 #include "List.h"
 #include "OpenCallbackConsole.h"
 #include "UpdateCallbackConsole.h"
 
-#include "HashCon.h"
-
-#ifdef PROG_VARIANT_R
+#ifdef Z7_PROG_VARIANT_R
 #include "../../../../C/7zVersion.h"
 #else
 #include "../../MyVersion.h"
@@ -54,7 +86,9 @@ using namespace NFile;
 using namespace NCommandLineParser;
 
 #ifdef _WIN32
-HINSTANCE g_hInstance = 0;
+extern
+HINSTANCE g_hInstance;
+HINSTANCE g_hInstance = NULL;
 #endif
 
 extern CStdOutStream *g_StdStream;
@@ -66,28 +100,38 @@ extern const CCodecInfo *g_Codecs[];
 extern unsigned g_NumHashers;
 extern const CHasherInfo *g_Hashers[];
 
-static const char * const kCopyrightString = "\n7-Zip"
-  #ifndef EXTERNAL_CODECS
-    #ifdef PROG_VARIANT_R
-      " (r)"
-    #else
-      " (a)"
-    #endif
-  #endif
+#ifdef Z7_EXTERNAL_CODECS
+extern
+const CExternalCodecs *g_ExternalCodecs_Ptr;
+const CExternalCodecs *g_ExternalCodecs_Ptr;
+#endif
 
+DECLARE_AND_SET_CLIENT_VERSION_VAR
+
+#if defined(Z7_PROG_VARIANT_Z)
+  #define PROG_POSTFIX      "z"
+  #define PROG_POSTFIX_2  " (z)"
+#elif defined(Z7_PROG_VARIANT_R)
+  #define PROG_POSTFIX      "r"
+  #define PROG_POSTFIX_2  " (r)"
+#elif defined(Z7_PROG_VARIANT_A) || !defined(Z7_EXTERNAL_CODECS)
+  #define PROG_POSTFIX      "a"
+  #define PROG_POSTFIX_2  " (a)"
+#else
+  #define PROG_POSTFIX    ""
+  #define PROG_POSTFIX_2  ""
+#endif
+
+
+static const char * const kCopyrightString = "\n7-Zip"
+  PROG_POSTFIX_2
   " " MY_VERSION_CPU
-  " : " MY_COPYRIGHT_DATE "\n\n";
+  " : " MY_COPYRIGHT_DATE "\n";
 
 static const char * const kHelpString =
     "Usage: 7z"
-#ifndef EXTERNAL_CODECS
-#ifdef PROG_VARIANT_R
-    "r"
-#else
-    "a"
-#endif
-#endif
-    " <command> [<switches>...] <archive_name> [<file_names>...]\n"
+    PROG_POSTFIX
+    " <command> [<switches>...] <archive_name> [<file_names>...] [@listfile]\n"
     "\n"
     "<Commands>\n"
     "  a : Add files to archive\n"
@@ -103,29 +147,35 @@ static const char * const kHelpString =
     "  x : eXtract files with full paths\n"
     "\n"
     "<Switches>\n"
-    "  -- : Stop switches parsing\n"
-    "  @listfile : set path to listfile that contains file names\n"
-    "  -ai[r[-|0]]{@listfile|!wildcard} : Include archives\n"
-    "  -ax[r[-|0]]{@listfile|!wildcard} : eXclude archives\n"
+    "  -- : Stop switches and @listfile parsing\n"
+    "  -ai[r[-|0]][m[-|2]][w[-]]{@listfile|!wildcard} : Include archives\n"
+    "  -ax[r[-|0]][m[-|2]][w[-]]{@listfile|!wildcard} : eXclude archives\n"
     "  -ao{a|s|t|u} : set Overwrite mode\n"
     "  -an : disable archive_name field\n"
     "  -bb[0-3] : set output log level\n"
     "  -bd : disable progress indicator\n"
     "  -bs{o|e|p}{0|1|2} : set output stream for output/error/progress line\n"
     "  -bt : show execution time statistics\n"
-    "  -i[r[-|0]]{@listfile|!wildcard} : Include filenames\n"
+    "  -i[r[-|0]][m[-|2]][w[-]]{@listfile|!wildcard} : Include filenames\n"
     "  -m{Parameters} : set compression Method\n"
     "    -mmt[N] : set number of CPU threads\n"
     "    -mx[N] : set compression level: -mx1 (fastest) ... -mx9 (ultra)\n"
     "  -o{Directory} : set Output directory\n"
-    #ifndef _NO_CRYPTO
+    #ifndef Z7_NO_CRYPTO
     "  -p{Password} : set Password\n"
     #endif
-    "  -r[-|0] : Recurse subdirectories\n"
+    "  -r[-|0] : Recurse subdirectories for name search\n"
     "  -sa{a|e|s} : set Archive name mode\n"
-    "  -scc{UTF-8|WIN|DOS} : set charset for for console input/output\n"
+    "  -scc{UTF-8|WIN|DOS} : set charset for console input/output\n"
     "  -scs{UTF-8|UTF-16LE|UTF-16BE|WIN|DOS|{id}} : set charset for list files\n"
-    "  -scrc[CRC32|CRC64|SHA1|SHA256|*] : set hash function for x, e, h commands\n"
+    "  -scrc[CRC32|CRC64|SHA256"
+#ifndef Z7_PROG_VARIANT_R
+    "|SHA1|XXH64"
+#ifdef Z7_PROG_VARIANT_Z
+    "|BLAKE2SP"
+#endif
+#endif
+    "|*] : set hash function for x, e, h commands\n"
     "  -sdel : delete files after compression\n"
     "  -seml[.] : send archive by email\n"
     "  -sfx[{name}] : Create SFX archive\n"
@@ -139,9 +189,10 @@ static const char * const kHelpString =
     "  -so : write data to stdout\n"
     "  -spd : disable wildcard matching for file names\n"
     "  -spe : eliminate duplication of root folder for extract command\n"
-    "  -spf : use fully qualified file paths\n"
+    "  -spf[2] : use fully qualified file paths\n"
     "  -ssc[-] : set sensitive case mode\n"
     "  -sse : stop archive creating, if it can't open some input file\n"
+    "  -ssp : do not change Last Access Time of source files while archiving\n"
     "  -ssw : compress shared files\n"
     "  -stl : set archive timestamp from the most recently modified file\n"
     "  -stm{HexMask} : set CPU thread affinity mask (hexadecimal number)\n"
@@ -150,7 +201,7 @@ static const char * const kHelpString =
     "  -u[-][p#][q#][r#][x#][y#][z#][!newArchiveName] : Update options\n"
     "  -v{Size}[b|k|m|g] : Create volumes\n"
     "  -w[{path}] : assign Work directory. Empty path means a temporary directory\n"
-    "  -x[r[-|0]]{@listfile|!wildcard} : eXclude filenames\n"
+    "  -x[r[-|0]][m[-|2]][w[-]]{@listfile|!wildcard} : eXclude filenames\n"
     "  -y : assume Yes on all queries\n";
 
 // ---------------------------
@@ -162,8 +213,11 @@ static const char * const kNoFormats = "7-Zip cannot find the code that works wi
 static const char * const kUnsupportedArcTypeMessage = "Unsupported archive type";
 // static const char * const kUnsupportedUpdateArcType = "Can't create archive for that type";
 
+#ifndef Z7_EXTRACT_ONLY
 #define kDefaultSfxModule "7zCon.sfx"
+#endif
 
+Z7_ATTR_NORETURN
 static void ShowMessageAndThrowException(LPCSTR message, NExitCode::EEnum code)
 {
   if (g_ErrStream)
@@ -171,15 +225,151 @@ static void ShowMessageAndThrowException(LPCSTR message, NExitCode::EEnum code)
   throw code;
 }
 
-#ifndef _WIN32
-static void GetArguments(int numArgs, const char *args[], UStringVector &parts)
+
+#ifdef _WIN32
+#define ShowProgInfo(so)
+#else
+static void ShowProgInfo(CStdOutStream *so)
 {
-  parts.Clear();
-  for (int i = 0; i < numArgs; i++)
+  if (!so)
+    return;
+
+  *so
+  
+  /*
+  #ifdef __DATE__
+      << " " << __DATE__
+  #endif
+  #ifdef __TIME__
+      << " " << __TIME__
+  #endif
+  */
+
+  << " " << (unsigned)(sizeof(void *)) * 8 << "-bit"
+
+  #ifdef __ILP32__
+    << " ILP32"
+  #endif
+
+  #ifdef __ARM_ARCH
+  << " arm_v:" << __ARM_ARCH
+  #if (__ARM_ARCH == 8)
+    // for macos:
+    #if   defined(__ARM_ARCH_8_9__)
+      << ".9"
+    #elif defined(__ARM_ARCH_8_8__)
+      << ".8"
+    #elif defined(__ARM_ARCH_8_7__)
+      << ".7"
+    #elif defined(__ARM_ARCH_8_6__)
+      << ".6"
+    #elif defined(__ARM_ARCH_8_5__)
+      << ".5"
+    #elif defined(__ARM_ARCH_8_4__)
+      << ".4"
+    #elif defined(__ARM_ARCH_8_3__)
+      << ".3"
+    #elif defined(__ARM_ARCH_8_2__)
+      << ".2"
+    #elif defined(__ARM_ARCH_8_1__)
+      << ".1"
+    #endif
+  #endif
+    
+    #if defined(__ARM_ARCH_PROFILE) && \
+        (   __ARM_ARCH_PROFILE >= 'A' && __ARM_ARCH_PROFILE <= 'Z' \
+         || __ARM_ARCH_PROFILE >= 65  && __ARM_ARCH_PROFILE <= 65 + 25)
+      << "-" << (char)__ARM_ARCH_PROFILE
+    #endif
+
+  #ifdef __ARM_ARCH_ISA_THUMB
+  << " thumb:" << __ARM_ARCH_ISA_THUMB
+  #endif
+  #endif
+
+  #ifdef _MIPS_ARCH
+  << " mips_arch:" << _MIPS_ARCH
+  #endif
+  #ifdef __mips_isa_rev
+  << " mips_isa_rev:" << __mips_isa_rev
+  #endif
+
+  #ifdef __iset__
+  << " e2k_v:" << __iset__
+  #endif
+  ;
+
+
+
+  #ifdef ENV_HAVE_LOCALE
+    *so << " locale=" << GetLocale();
+  #endif
+  #ifndef _WIN32
   {
-    UString s = MultiByteToUnicodeString(args[i]);
-    parts.Add(s);
+    const bool is_IsNativeUTF8 = IsNativeUTF8();
+    if (!is_IsNativeUTF8)
+      *so << " UTF8=" << (is_IsNativeUTF8 ? "+" : "-");
   }
+  if (!g_ForceToUTF8)
+    *so << " use-UTF8=" << (g_ForceToUTF8 ? "+" : "-");
+  {
+    const unsigned wchar_t_size = (unsigned)sizeof(wchar_t);
+    if (wchar_t_size != 4)
+      *so << " wchar_t=" << wchar_t_size * 8 << "-bit";
+  }
+  {
+    const unsigned off_t_size = (unsigned)sizeof(off_t);
+    if (off_t_size != 8)
+      *so << " Files=" << off_t_size * 8 << "-bit";
+  }
+  #endif
+  
+  {
+    const UInt32 numCpus = NWindows::NSystem::GetNumberOfProcessors();
+    *so << " Threads:" << numCpus;
+    const UInt64 openMAX= NWindows::NSystem::Get_File_OPEN_MAX();
+    *so << " OPEN_MAX:" << openMAX;
+    {
+      FString temp;
+      NDir::MyGetTempPath(temp);
+      if (!temp.IsEqualTo(STRING_PATH_SEPARATOR "tmp" STRING_PATH_SEPARATOR))
+        *so << " temp_path:" << temp;
+    }
+  }
+
+  #ifdef Z7_7ZIP_ASM
+  *so << ", ASM";
+  #endif
+
+  /*
+  {
+    AString s;
+    GetCpuName(s);
+    s.Trim();
+    *so << ", " << s;
+  }
+
+  #ifdef __ARM_FEATURE_CRC32
+     << " CRC32"
+  #endif
+
+  
+  #if (defined MY_CPU_X86_OR_AMD64 || defined(MY_CPU_ARM_OR_ARM64))
+  if (CPU_IsSupported_AES()) *so << ",AES";
+  #endif
+  
+  #ifdef MY_CPU_ARM_OR_ARM64
+  if (CPU_IsSupported_CRC32()) *so << ",CRC32";
+  #if defined(_WIN32)
+  if (CPU_IsSupported_CRYPTO()) *so << ",CRYPTO";
+  #else
+  if (CPU_IsSupported_SHA1()) *so << ",SHA1";
+  if (CPU_IsSupported_SHA2()) *so << ",SHA2";
+  #endif
+  #endif
+  */
+
+  *so << endl;
 }
 #endif
 
@@ -189,6 +379,8 @@ static void ShowCopyrightAndHelp(CStdOutStream *so, bool needHelp)
     return;
   *so << kCopyrightString;
   // *so << "# CPUs: " << (UInt64)NWindows::NSystem::GetNumberOfProcessors() << endl;
+  ShowProgInfo(so);
+  *so << endl;
   if (needHelp)
     *so << kHelpString;
 }
@@ -209,10 +401,21 @@ static void PrintUInt32(CStdOutStream &so, UInt32 val, unsigned size)
   PrintStringRight(so, s, size);
 }
 
+#ifdef Z7_EXTERNAL_CODECS
+static void PrintNumber(CStdOutStream &so, UInt32 val, unsigned numDigits)
+{
+  AString s;
+  s.Add_UInt32(val);
+  while (s.Len() < numDigits)
+    s.InsertAtFront('0');
+  so << s;
+}
+#endif
+
 static void PrintLibIndex(CStdOutStream &so, int libIndex)
 {
   if (libIndex >= 0)
-    PrintUInt32(so, libIndex, 2);
+    PrintUInt32(so, (UInt32)libIndex, 2);
   else
     so << "  ";
   so << ' ';
@@ -226,16 +429,11 @@ static void PrintString(CStdOutStream &so, const UString &s, unsigned size)
     so << ' ';
 }
 
-static inline char GetHex(unsigned val)
-{
-  return (char)((val < 10) ? ('0' + val) : ('A' + (val - 10)));
-}
-
 static void PrintWarningsPaths(const CErrorPathCodes &pc, CStdOutStream &so)
 {
   FOR_VECTOR(i, pc.Paths)
   {
-    so.NormalizePrint_UString(fs2us(pc.Paths[i]));
+    so.NormalizePrint_UString_Path(fs2us(pc.Paths[i]));
     so << " : ";
     so << NError::MyFormatMessage(pc.Codes[i]) << endl;
   }
@@ -330,7 +528,6 @@ static void ThrowException_if_Error(HRESULT res)
     throw CSystemException(res);
 }
 
-
 static void PrintNum(UInt64 val, unsigned numDigits, char c = ' ')
 {
   char temp[64];
@@ -342,6 +539,8 @@ static void PrintNum(UInt64 val, unsigned numDigits, char c = ' ')
   *g_StdStream << p;
 }
 
+#ifdef _WIN32
+
 static void PrintTime(const char *s, UInt64 val, UInt64 total)
 {
   *g_StdStream << endl << s << " Time =";
@@ -375,8 +574,12 @@ static void PrintMemUsage(const char *s, UInt64 val)
   *g_StdStream << "    " << s << " Memory =";
   PrintNum(SHIFT_SIZE_VALUE(val, 20), 7);
   *g_StdStream << " MB";
-
-  #ifdef _7ZIP_LARGE_PAGES
+  /*
+  *g_StdStream << " =";
+  PrintNum(SHIFT_SIZE_VALUE(val, 10), 9);
+  *g_StdStream << " KB";
+  */
+  #ifdef Z7_LARGE_PAGES
   AString lp;
   Add_LargePages_String(lp);
   if (!lp.IsEmpty())
@@ -407,10 +610,12 @@ static void PrintStat()
       , &creationTimeFT, &exitTimeFT, &kernelTimeFT, &userTimeFT))
     return;
   FILETIME curTimeFT;
-  NTime::GetCurUtcFileTime(curTimeFT);
+  NTime::GetCurUtc_FiTime(curTimeFT);
 
   #ifndef UNDER_CE
   
+Z7_DIAGNOSTIC_IGNORE_CAST_FUNCTION
+
   PROCESS_MEMORY_COUNTERS m;
   memset(&m, 0, sizeof(m));
   BOOL memDefined = FALSE;
@@ -425,22 +630,27 @@ static void PrintStat()
        The program with K32GetProcessMemoryInfo will not work on systems before Win7
        // memDefined = GetProcessMemoryInfo(GetCurrentProcess(), &m, sizeof(m));
     */
-
-    HMODULE kern = ::GetModuleHandleW(L"kernel32.dll");
-    Func_GetProcessMemoryInfo my_GetProcessMemoryInfo = (Func_GetProcessMemoryInfo)
-        ::GetProcAddress(kern, "K32GetProcessMemoryInfo");
+    const HMODULE kern = ::GetModuleHandleW(L"kernel32.dll");
+    Func_GetProcessMemoryInfo
+      my_GetProcessMemoryInfo = Z7_GET_PROC_ADDRESS(
+    Func_GetProcessMemoryInfo, kern,
+     "K32GetProcessMemoryInfo");
     if (!my_GetProcessMemoryInfo)
     {
-      HMODULE lib = LoadLibraryW(L"Psapi.dll");
+      const HMODULE lib = LoadLibraryW(L"Psapi.dll");
       if (lib)
-        my_GetProcessMemoryInfo = (Func_GetProcessMemoryInfo)::GetProcAddress(lib, "GetProcessMemoryInfo");
+          my_GetProcessMemoryInfo = Z7_GET_PROC_ADDRESS(
+        Func_GetProcessMemoryInfo, lib,
+            "GetProcessMemoryInfo");
     }
     if (my_GetProcessMemoryInfo)
       memDefined = my_GetProcessMemoryInfo(GetCurrentProcess(), &m, sizeof(m));
     // FreeLibrary(lib);
-
-    Func_QueryProcessCycleTime my_QueryProcessCycleTime = (Func_QueryProcessCycleTime)
-        ::GetProcAddress(kern, "QueryProcessCycleTime");
+    const
+    Func_QueryProcessCycleTime
+      my_QueryProcessCycleTime = Z7_GET_PROC_ADDRESS(
+    Func_QueryProcessCycleTime, kern,
+        "QueryProcessCycleTime");
     if (my_QueryProcessCycleTime)
       cycleDefined = my_QueryProcessCycleTime(GetCurrentProcess(), &cycleTime);
   }
@@ -456,18 +666,32 @@ static void PrintStat()
   
   PrintTime("Kernel ", kernelTime, totalTime);
 
+  const UInt64 processTime = kernelTime + userTime;
+  
   #ifndef UNDER_CE
   if (cycleDefined)
   {
-    *g_StdStream << " ";
-    PrintNum(cycleTime / 1000000, 22);
+    *g_StdStream << "    Cnt:";
+    PrintNum(cycleTime / 1000000, 15);
     *g_StdStream << " MCycles";
   }
   #endif
 
   PrintTime("User   ", userTime, totalTime);
+
+  #ifndef UNDER_CE
+  if (cycleDefined)
+  {
+    *g_StdStream << "    Freq (cnt/ptime):";
+    UInt64 us = processTime / 10;
+    if (us == 0)
+      us = 1;
+    PrintNum(cycleTime / us, 6);
+    *g_StdStream << " MHz";
+  }
+  #endif
   
-  PrintTime("Process", kernelTime + userTime, totalTime);
+  PrintTime("Process", processTime, totalTime);
   #ifndef UNDER_CE
   if (memDefined) PrintMemUsage("Virtual ", m.PeakPagefileUsage);
   #endif
@@ -476,10 +700,97 @@ static void PrintStat()
   #ifndef UNDER_CE
   if (memDefined) PrintMemUsage("Physical", m.PeakWorkingSetSize);
   #endif
-  
   *g_StdStream << endl;
 }
 
+
+#else  // ! _WIN32
+
+static UInt64 Get_timeofday_us()
+{
+  struct timeval now;
+  if (gettimeofday(&now, NULL) == 0)
+    return (UInt64)now.tv_sec * 1000000 + (UInt64)now.tv_usec;
+  return 0;
+}
+
+static void PrintTime(const char *s, UInt64 val, UInt64 total_us, UInt64 kFreq)
+{
+  *g_StdStream << endl << s << " Time =";
+
+  {
+    UInt64 sec, ms;
+    
+    if (kFreq == 0)
+    {
+      sec = val / 1000000;
+      ms  = val % 1000000 / 1000;
+    }
+    else
+    {
+      sec = val / kFreq;
+      ms = (UInt32)((val - (sec * kFreq)) * 1000 / kFreq);
+    }
+    
+    PrintNum(sec, 6);
+    *g_StdStream << '.';
+    PrintNum(ms, 3, '0');
+  }
+  
+  if (total_us == 0)
+    return;
+
+  UInt64 percent = 0;
+  if (kFreq == 0)
+    percent = val * 100 / total_us;
+  else
+  {
+    const UInt64 kMaxVal = (UInt64)(Int64)-1;
+    UInt32 m = 100000000;
+    for (;;)
+    {
+      if (m == 0 || kFreq == 0)
+        break;
+      if (kMaxVal / m > val &&
+        kMaxVal / kFreq > total_us)
+        break;
+      if (val > m)
+        val >>= 1;
+      else
+        m >>= 1;
+      if (kFreq > total_us)
+        kFreq >>= 1;
+      else
+        total_us >>= 1;
+    }
+    const UInt64 total = kFreq * total_us;
+    if (total != 0)
+      percent = val * m / total;
+  }
+  *g_StdStream << " =";
+  PrintNum(percent, 5);
+  *g_StdStream << '%';
+}
+
+static void PrintStat(const UInt64 startTime)
+{
+  tms t;
+  /* clock_t res = */ times(&t);
+  const UInt64 totalTime = Get_timeofday_us() - startTime;
+  const UInt64 kFreq = (UInt64)sysconf(_SC_CLK_TCK);
+  PrintTime("Kernel ", (UInt64)t.tms_stime, totalTime, kFreq);
+  PrintTime("User   ", (UInt64)t.tms_utime, totalTime, kFreq);
+  PrintTime("Process", (UInt64)t.tms_utime + (UInt64)t.tms_stime, totalTime, kFreq);
+  PrintTime("Global ", totalTime, totalTime, 0);
+  *g_StdStream << endl;
+}
+
+#endif // ! _WIN32
+
+
+
+
+
 static void PrintHexId(CStdOutStream &so, UInt64 id)
 {
   char s[32];
@@ -487,23 +798,76 @@ static void PrintHexId(CStdOutStream &so, UInt64 id)
   PrintStringRight(so, s, 8);
 }
 
+#ifndef _WIN32
+void Set_ModuleDirPrefix_From_ProgArg0(const char *s);
+#endif
 
+int Main2(
+  #ifndef _WIN32
+  int numArgs, char *args[]
+  #endif
+);
 int Main2(
   #ifndef _WIN32
   int numArgs, char *args[]
   #endif
 )
 {
+  #if defined(MY_CPU_SIZEOF_POINTER)
+    { unsigned k = sizeof(void *); if (k != MY_CPU_SIZEOF_POINTER) throw "incorrect MY_CPU_PTR_SIZE"; }
+  #endif
+
   #if defined(_WIN32) && !defined(UNDER_CE)
   SetFileApisToOEM();
   #endif
 
+  #ifdef ENV_HAVE_LOCALE
+  // printf("\nBefore SetLocale() : %s\n", IsNativeUtf8() ? "NATIVE UTF-8" : "IS NOT NATIVE UTF-8");
+  MY_SetLocale();
+  // printf("\nAfter  SetLocale() : %s\n", IsNativeUtf8() ? "NATIVE UTF-8" : "IS NOT NATIVE UTF-8");
+  #endif
+
+  #ifndef _WIN32
+  const UInt64 startTime = Get_timeofday_us();
+  #endif
+
+  /*
+  {
+    g_StdOut << "DWORD:" << (unsigned)sizeof(DWORD);
+    g_StdOut << " LONG:" << (unsigned)sizeof(LONG);
+    g_StdOut << " long:" << (unsigned)sizeof(long);
+    #ifdef _WIN64
+    // g_StdOut << " long long:" << (unsigned)sizeof(long long);
+    #endif
+    g_StdOut << " int:" << (unsigned)sizeof(int);
+    g_StdOut << " void*:"  << (unsigned)sizeof(void *);
+    g_StdOut << endl;
+  }
+  */
+
   UStringVector commandStrings;
   
   #ifdef _WIN32
   NCommandLineParser::SplitCommandLine(GetCommandLineW(), commandStrings);
   #else
-  GetArguments(numArgs, args, commandStrings);
+  {
+    if (numArgs > 0)
+      Set_ModuleDirPrefix_From_ProgArg0(args[0]);
+
+    for (int i = 0; i < numArgs; i++)
+    {
+      AString a (args[i]);
+#if 0
+      printf("\n%d %s :", i, a.Ptr());
+      for (unsigned k = 0; k < a.Len(); k++)
+        printf(" %2x", (unsigned)(Byte)a[k]);
+#endif
+      const UString s = MultiByteToUnicodeString(a);
+      commandStrings.Add(s);
+    }
+    // printf("\n");
+  }
+
   #endif
 
   #ifndef UNDER_CE
@@ -534,7 +898,7 @@ int Main2(
 
   CStdOutStream *percentsStream = NULL;
   if (options.Number_for_Percents != k_OutStream_disabled)
-    percentsStream = (options.Number_for_Percents == k_OutStream_stderr) ? &g_StdErr : &g_StdOut;;
+    percentsStream = (options.Number_for_Percents == k_OutStream_stderr) ? &g_StdErr : &g_StdOut;
   
   if (options.HelpMode)
   {
@@ -543,10 +907,55 @@ int Main2(
   }
 
   if (options.EnableHeaders)
+  {
     ShowCopyrightAndHelp(g_StdStream, false);
+    if (!parser.Parse1Log.IsEmpty())
+      *g_StdStream << parser.Parse1Log;
+  }
 
   parser.Parse2(options);
 
+  {
+    int cp = options.ConsoleCodePage;
+    
+    int stdout_cp = cp;
+    int stderr_cp = cp;
+    int stdin_cp = cp;
+
+    /*
+    // these cases are complicated.
+    // maybe we must use CRT functions instead of console WIN32.
+    // different Windows/CRT versions also can work different ways.
+    // so the following code was not enabled:
+    if (cp == -1)
+    {
+      // we set CodePage only if stream is attached to terminal
+      // maybe we should set CodePage even if is not terminal?
+      #ifdef _WIN32
+      {
+        UINT ccp = GetConsoleOutputCP();
+        if (ccp != 0)
+        {
+          if (options.IsStdOutTerminal) stdout_cp = ccp;
+          if (options.IsStdErrTerminal) stderr_cp = ccp;
+        }
+      }
+      if (options.IsInTerminal)
+      {
+        UINT ccp = GetConsoleCP();
+        if (ccp != 0) stdin_cp = ccp;
+      }
+      #endif
+    }
+    */
+    
+    if (stdout_cp != -1) g_StdOut.CodePage = stdout_cp;
+    if (stderr_cp != -1) g_StdErr.CodePage = stderr_cp;
+    if (stdin_cp != -1) g_StdIn.CodePage = stdin_cp;
+  }
+  g_StdOut.ListPathSeparatorSlash = options.ListPathSeparatorSlash;
+  g_StdErr.ListPathSeparatorSlash = options.ListPathSeparatorSlash;
+
   unsigned percentsNameLevel = 1;
   if (options.LogLevel == 0 || options.Number_for_Percents != options.Number_for_Out)
     percentsNameLevel = 2;
@@ -560,32 +969,47 @@ int Main2(
     #if !defined(UNDER_CE)
     CONSOLE_SCREEN_BUFFER_INFO consoleInfo;
     if (GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &consoleInfo))
-      consoleWidth = consoleInfo.dwSize.X;
+      consoleWidth = (USHORT)consoleInfo.dwSize.X;
     #endif
     
     #else
     
+#if !defined(__sun)
     struct winsize w;
-    if (ioctl(0, TIOCGWINSZ, &w) == )
+    if (ioctl(0, TIOCGWINSZ, &w) == 0)
       consoleWidth = w.ws_col;
-    
+#endif
     #endif
   }
 
   CREATE_CODECS_OBJECT
 
-  codecs->CaseSensitiveChange = options.CaseSensitiveChange;
+  codecs->CaseSensitive_Change = options.CaseSensitive_Change;
   codecs->CaseSensitive = options.CaseSensitive;
   ThrowException_if_Error(codecs->Load());
+  Codecs_AddHashArcHandler(codecs);
 
-  bool isExtractGroupCommand = options.Command.IsFromExtractGroup();
+  #ifdef Z7_EXTERNAL_CODECS
+  {
+    g_ExternalCodecs_Ptr = &_externalCodecs;
+    UString s;
+    codecs->GetCodecsErrorMessage(s);
+    if (!s.IsEmpty())
+    {
+      CStdOutStream &so = (g_StdStream ? *g_StdStream : g_StdOut);
+      so << endl << s << endl;
+    }
+  }
+  #endif
+
+  const bool isExtractGroupCommand = options.Command.IsFromExtractGroup();
 
   if (codecs->Formats.Size() == 0 &&
         (isExtractGroupCommand
         || options.Command.CommandType == NCommandType::kList
         || options.Command.IsFromUpdateGroup()))
   {
-    #ifdef EXTERNAL_CODECS
+    #ifdef Z7_EXTERNAL_CODECS
     if (!codecs->MainDll_ErrorPath.IsEmpty())
     {
       UString s ("Can't load module: ");
@@ -593,13 +1017,15 @@ int Main2(
       throw s;
     }
     #endif
-    
     throw kNoFormats;
   }
 
   CObjectVector<COpenType> types;
   if (!ParseOpenTypes(*codecs, options.ArcType, types))
+  {
     throw kUnsupportedArcTypeMessage;
+  }
+
 
   CIntVector excludedFormats;
   FOR_VECTOR (k, options.ExcludedArcTypes)
@@ -608,16 +1034,19 @@ int Main2(
     if (!codecs->FindFormatForArchiveType(options.ExcludedArcTypes[k], tempIndices)
         || tempIndices.Size() != 1)
       throw kUnsupportedArcTypeMessage;
+    
+    
+    
     excludedFormats.AddToUniqueSorted(tempIndices[0]);
     // excludedFormats.Sort();
   }
-
   
-  #ifdef EXTERNAL_CODECS
+  #ifdef Z7_EXTERNAL_CODECS
   if (isExtractGroupCommand
+      || options.Command.IsFromUpdateGroup()
       || options.Command.CommandType == NCommandType::kHash
       || options.Command.CommandType == NCommandType::kBenchmark)
-    ThrowException_if_Error(__externalCodecs.Load());
+    ThrowException_if_Error(_externalCodecs.Load());
   #endif
 
   int retCode = NExitCode::kSuccess;
@@ -637,38 +1066,54 @@ int Main2(
     CStdOutStream &so = (g_StdStream ? *g_StdStream : g_StdOut);
     unsigned i;
 
-    #ifdef EXTERNAL_CODECS
+    #ifdef Z7_EXTERNAL_CODECS
     so << endl << "Libs:" << endl;
     for (i = 0; i < codecs->Libs.Size(); i++)
     {
-      PrintLibIndex(so, i);
-      so << ' ' << codecs->Libs[i].Path << endl;
+      PrintLibIndex(so, (int)i);
+      const CCodecLib &lib = codecs->Libs[i];
+      // if (lib.Version != 0)
+      so << ": " << (lib.Version >> 16) << ".";
+      PrintNumber(so, lib.Version & 0xffff, 2);
+      so << " : " << lib.Path << endl;
     }
     #endif
 
     so << endl << "Formats:" << endl;
     
-    const char * const kArcFlags = "KSNFMGOPBELH";
+    const char * const kArcFlags = "KSNFMGOPBELHXCc+a+m+r+";
+    const char * const kArcTimeFlags = "wudn";
     const unsigned kNumArcFlags = (unsigned)strlen(kArcFlags);
-    
+    const unsigned kNumArcTimeFlags = (unsigned)strlen(kArcTimeFlags);
+
     for (i = 0; i < codecs->Formats.Size(); i++)
     {
       const CArcInfoEx &arc = codecs->Formats[i];
 
-      #ifdef EXTERNAL_CODECS
+      #ifdef Z7_EXTERNAL_CODECS
       PrintLibIndex(so, arc.LibIndex);
       #else
-      so << "  ";
+      so << "   ";
       #endif
 
       so << (char)(arc.UpdateEnabled ? 'C' : ' ');
       
-      for (unsigned b = 0; b < kNumArcFlags; b++)
       {
-        so << (char)
-          ((arc.Flags & ((UInt32)1 << b)) != 0 ? kArcFlags[b] : ' ');
+        unsigned b;
+        for (b = 0; b < kNumArcFlags; b++)
+          so << (char)((arc.Flags & ((UInt32)1 << b)) != 0 ? kArcFlags[b] : '.');
+        so << ' ';
       }
-        
+
+      if (arc.TimeFlags != 0)
+      {
+        unsigned b;
+        for (b = 0; b < kNumArcTimeFlags; b++)
+          so << (char)((arc.TimeFlags & ((UInt32)1 << b)) != 0 ? kArcTimeFlags[b] : '.');
+        so << arc.Get_DefaultTimePrec();
+        so << ' ';
+      }
+      
       so << ' ';
       PrintString(so, arc.Name, 8);
       so << ' ';
@@ -684,7 +1129,7 @@ int Main2(
         {
           s += " (";
           s += ext.AddExt;
-          s += ')';
+          s.Add_Char(')');
         }
       }
       
@@ -694,6 +1139,8 @@ int Main2(
       if (arc.SignatureOffset != 0)
         so << "offset=" << arc.SignatureOffset << ' ';
 
+      // so << "numSignatures = " << arc.Signatures.Size() << " ";
+
       FOR_VECTOR(si, arc.Signatures)
       {
         if (si != 0)
@@ -705,15 +1152,15 @@ int Main2(
         {
           if (j != 0)
             so << ' ';
-          Byte b = sig[j];
+          const unsigned b = sig.ConstData()[j];
           if (b > 0x20 && b < 0x80)
           {
             so << (char)b;
           }
           else
           {
-            so << GetHex((b >> 4) & 0xF);
-            so << GetHex(b & 0xF);
+            so << GET_HEX_CHAR_UPPER(b >> 4);
+            so << GET_HEX_CHAR_UPPER(b & 15);
           }
         }
       }
@@ -735,6 +1182,7 @@ int Main2(
       
       so << (char)(cod.CreateEncoder ? 'E' : ' ');
       so << (char)(cod.CreateDecoder ? 'D' : ' ');
+      so << (char)(cod.IsFilter      ? 'F' : ' ');
 
       so << ' ';
       PrintHexId(so, cod.Id);
@@ -742,10 +1190,10 @@ int Main2(
     }
 
 
-    #ifdef EXTERNAL_CODECS
+    #ifdef Z7_EXTERNAL_CODECS
 
     UInt32 numMethods;
-    if (codecs->GetNumMethods(&numMethods) == S_OK)
+    if (_externalCodecs.GetCodecs->GetNumMethods(&numMethods) == S_OK)
     for (UInt32 j = 0; j < numMethods; j++)
     {
       PrintLibIndex(so, codecs->GetCodec_LibIndex(j));
@@ -758,6 +1206,12 @@ int Main2(
       
       so << (char)(codecs->GetCodec_EncoderIsAssigned(j) ? 'E' : ' ');
       so << (char)(codecs->GetCodec_DecoderIsAssigned(j) ? 'D' : ' ');
+      {
+        bool isFilter_Assigned;
+        const bool isFilter = codecs->GetCodec_IsFilter(j, isFilter_Assigned);
+        so << (char)(isFilter ? 'F' : isFilter_Assigned ? ' ' : '*');
+      }
+
 
       so << ' ';
       UInt64 id;
@@ -783,9 +1237,9 @@ int Main2(
       so << ' ' << codec.Name << endl;
     }
 
-    #ifdef EXTERNAL_CODECS
+    #ifdef Z7_EXTERNAL_CODECS
     
-    numMethods = codecs->GetNumHashers();
+    numMethods = _externalCodecs.GetHashers->GetNumHashers();
     for (UInt32 j = 0; j < numMethods; j++)
     {
       PrintLibIndex(so, codecs->GetHasherLibIndex(j));
@@ -805,6 +1259,7 @@ int Main2(
         options.Properties, options.NumIterations, (FILE *)so);
     if (hresultMain == S_FALSE)
     {
+      so << endl;
       if (g_ErrStream)
         *g_ErrStream << "\nDecoding ERROR\n";
       retCode = NExitCode::kFatalError;
@@ -864,13 +1319,13 @@ int Main2(
       }
     }
 
-    if (hresultMain == S_OK)
+    if (hresultMain == S_OK) {
     if (isExtractGroupCommand)
     {
       CExtractCallbackConsole *ecs = new CExtractCallbackConsole;
       CMyComPtr<IFolderArchiveExtractCallback> extractCallback = ecs;
 
-      #ifndef _NO_CRYPTO
+      #ifndef Z7_NO_CRYPTO
       ecs->PasswordIsDefined = options.PasswordEnabled;
       ecs->Password = options.Password;
       #endif
@@ -888,7 +1343,7 @@ int Main2(
       COpenCallbackConsole openCallback;
       openCallback.Init(g_StdStream, g_ErrStream);
 
-      #ifndef _NO_CRYPTO
+      #ifndef Z7_NO_CRYPTO
       openCallback.PasswordIsDefined = options.PasswordEnabled;
       openCallback.Password = options.Password;
       #endif
@@ -902,7 +1357,7 @@ int Main2(
       eo.YesToAll = options.YesToAll;
       eo.TestMode = options.Command.IsTestCommand();
       
-      #ifndef _SFX
+      #ifndef Z7_SFX
       eo.Properties = options.Properties;
       #endif
 
@@ -919,13 +1374,16 @@ int Main2(
       }
       
       hresultMain = Extract(
+          // EXTERNAL_CODECS_VARS_L
           codecs,
           types,
           excludedFormats,
           ArchivePathsSorted,
           ArchivePathsFullSorted,
           options.Censor.Pairs.Front().Head,
-          eo, ecs, ecs, hashCalc, errorMessage, stat);
+          eo,
+          ecs, ecs, ecs,
+          hashCalc, errorMessage, stat);
       
       ecs->ClosePercents();
 
@@ -993,7 +1451,7 @@ int Main2(
       if (isError)
         retCode = NExitCode::kFatalError;
       
-      if (so)
+      if (so) {
       if (ecs->NumArcsWithError != 0 || ecs->NumFileErrors != 0)
       {
         // if (ecs->NumArchives > 1)
@@ -1024,15 +1482,21 @@ int Main2(
           PrintHashStat(*so, hb);
         }
       }
+      } // if (so)
     }
-    else
+    else // if_(!isExtractGroupCommand)
     {
       UInt64 numErrors = 0;
       UInt64 numWarnings = 0;
       
       // options.ExtractNtOptions.StoreAltStreams = true, if -sns[-] is not definmed
 
+      CListOptions lo;
+      lo.ExcludeDirItems = options.Censor.ExcludeDirItems;
+      lo.ExcludeFileItems = options.Censor.ExcludeFileItems;
+
       hresultMain = ListArchives(
+          lo,
           codecs,
           types,
           excludedFormats,
@@ -1044,7 +1508,7 @@ int Main2(
           options.Censor.Pairs.Front().Head,
           options.EnableHeaders,
           options.TechMode,
-          #ifndef _NO_CRYPTO
+          #ifndef Z7_NO_CRYPTO
           options.PasswordEnabled,
           options.Password,
           #endif
@@ -1061,10 +1525,14 @@ int Main2(
           g_StdOut << endl << "Errors: " << numErrors << endl;
         retCode = NExitCode::kFatalError;
       }
-    }
+    } // if_(isExtractGroupCommand)
+    } // if_(hresultMain == S_OK)
   }
   else if (options.Command.IsFromUpdateGroup())
   {
+   #ifdef Z7_EXTRACT_ONLY
+    throw "update commands are not implemented";
+   #else
     CUpdateOptions &uo = options.UpdateOptions;
     if (uo.SfxMode && uo.SfxModule.IsEmpty())
       uo.SfxModule = kDefaultSfxModule;
@@ -1072,7 +1540,7 @@ int Main2(
     COpenCallbackConsole openCallback;
     openCallback.Init(g_StdStream, g_ErrStream, percentsStream);
 
-    #ifndef _NO_CRYPTO
+    #ifndef Z7_NO_CRYPTO
     bool passwordIsDefined =
         (options.PasswordEnabled && !options.Password.IsEmpty());
     openCallback.PasswordIsDefined = passwordIsDefined;
@@ -1086,7 +1554,7 @@ int Main2(
     if (percentsStream)
       callback.SetWindowWidth(consoleWidth);
 
-    #ifndef _NO_CRYPTO
+    #ifndef Z7_NO_CRYPTO
     callback.PasswordIsDefined = passwordIsDefined;
     callback.AskPassword = (options.PasswordEnabled && options.Password.IsEmpty());
     callback.Password = options.Password;
@@ -1120,6 +1588,7 @@ int Main2(
         g_StdStream, se,
         true // options.EnableHeaders
         );
+   #endif
   }
   else if (options.Command.CommandType == NCommandType::kHash)
   {
@@ -1131,6 +1600,7 @@ int Main2(
   
     callback.Init(g_StdStream, g_ErrStream, percentsStream);
     callback.PrintHeaders = options.EnableHeaders;
+    callback.PrintFields = options.ListFields;
 
     AString errorInfoString;
     hresultMain = HashCalc(EXTERNAL_CODECS_VARS_L
@@ -1147,7 +1617,11 @@ int Main2(
     ShowMessageAndThrowException(kUserErrorMessage, NExitCode::kUserError);
 
   if (options.ShowTime && g_StdStream)
-    PrintStat();
+    PrintStat(
+      #ifndef _WIN32
+        startTime
+      #endif
+    );
 
   ThrowException_if_Error(hresultMain);
 
