@@ -292,6 +292,10 @@ class SyncConnection {
 
     bool IsValid() { return fd >= 0; }
 
+    void SetQuiet(bool quiet) {
+        line_printer_.quiet_ = quiet;
+    }
+
     void NewTransfer() {
         current_ledger_.Reset();
     }
@@ -811,30 +815,45 @@ class SyncConnection {
         while (!deferred_acknowledgements_.empty()) {
             bool should_block = read_all || deferred_acknowledgements_.size() >= max_deferred_acks;
 
-            ssize_t rc = adb_poll(&pfd, 1, should_block ? -1 : 0);
-            if (rc == 0) {
-                CHECK(!should_block);
-                return true;
-            }
+            enum class ReadStatus {
+                Success,
+                Failure,
+                TryLater,
+            };
 
-            if (acknowledgement_buffer_.size() < sizeof(sync_status)) {
-                const ssize_t header_bytes_left = sizeof(sync_status) - buf.size();
-                ssize_t rc = adb_read(fd, buf.end(), header_bytes_left);
-                if (rc <= 0) {
-                    Error("failed to read copy response");
-                    return false;
+            // Read until the acknowledgement buffer has at least `amount` bytes in it (or there is
+            // an I/O error, or the socket has no data ready to read).
+            auto read_until_amount = [&](size_t amount) -> ReadStatus {
+                while (buf.size() < amount) {
+                    // The fd is blocking, so if we want to avoid blocking in this function, we must
+                    // poll first to verify that some data is available before trying to read it.
+                    if (!should_block) {
+                        ssize_t rc = adb_poll(&pfd, 1, 0);
+                        if (rc == 0) {
+                            return ReadStatus::TryLater;
+                        }
+                    }
+                    const ssize_t bytes_left = amount - buf.size();
+                    ssize_t rc = adb_read(fd, buf.end(), bytes_left);
+                    if (rc <= 0) {
+                        Error("failed to read copy response");
+                        return ReadStatus::Failure;
+                    }
+                    buf.resize(buf.size() + rc);
+                    if (!should_block && buf.size() < amount) {
+                        return ReadStatus::TryLater;
+                    }
                 }
+                return ReadStatus::Success;
+            };
 
-                buf.resize(buf.size() + rc);
-                if (rc != header_bytes_left) {
-                    // Early exit if we run out of data in the socket.
+            switch (read_until_amount(sizeof(sync_status))) {
+                case ReadStatus::TryLater:
                     return true;
-                }
-
-                if (!should_block) {
-                    // We don't want to read again yet, because the socket might be empty.
-                    continue;
-                }
+                case ReadStatus::Failure:
+                    return false;
+                case ReadStatus::Success:
+                    break;
             }
 
             auto* hdr = reinterpret_cast<sync_status*>(buf.data());
@@ -854,29 +873,19 @@ class SyncConnection {
                 return false;
             }
 
-            const ssize_t msg_bytes_left = hdr->msglen + sizeof(sync_status) - buf.size();
-            CHECK_GE(msg_bytes_left, 0);
-            if (msg_bytes_left > 0) {
-                ssize_t rc = adb_read(fd, buf.end(), msg_bytes_left);
-                if (rc <= 0) {
-                    Error("failed to read copy failure message");
+            switch (read_until_amount(sizeof(sync_status) + hdr->msglen)) {
+                case ReadStatus::TryLater:
+                    return true;
+                case ReadStatus::Failure:
                     return false;
-                }
-
-                buf.resize(buf.size() + rc);
-                if (rc != msg_bytes_left) {
-                    if (should_block) {
-                        continue;
-                    } else {
-                        return true;
-                    }
-                }
-
-                std::string msg(buf.begin() + sizeof(sync_status), buf.end());
-                ReportDeferredCopyFailure(msg);
-                buf.resize(0);
-                return false;
+                case ReadStatus::Success:
+                    break;
             }
+
+            std::string msg(buf.begin() + sizeof(sync_status), buf.end());
+            ReportDeferredCopyFailure(msg);
+            buf.resize(0);
+            return false;
         }
 
         return true;
@@ -1437,9 +1446,10 @@ static bool copy_local_dir_remote(SyncConnection& sc, std::string lpath, std::st
 }
 
 bool do_sync_push(const std::vector<const char*>& srcs, const char* dst, bool sync,
-                  CompressionType compression, bool dry_run) {
+                  CompressionType compression, bool dry_run, bool quiet) {
     SyncConnection sc;
     if (!sc.IsValid()) return false;
+    sc.SetQuiet(quiet);
 
     bool success = true;
     bool dst_exists;
@@ -1659,9 +1669,10 @@ static bool copy_remote_dir_local(SyncConnection& sc, std::string rpath, std::st
 }
 
 bool do_sync_pull(const std::vector<const char*>& srcs, const char* dst, bool copy_attrs,
-                  CompressionType compression, const char* name) {
+                  CompressionType compression, const char* name, bool quiet) {
     SyncConnection sc;
     if (!sc.IsValid()) return false;
+    sc.SetQuiet(quiet);
 
     bool success = true;
     struct stat st;
@@ -1769,9 +1780,10 @@ bool do_sync_pull(const std::vector<const char*>& srcs, const char* dst, bool co
 }
 
 bool do_sync_sync(const std::string& lpath, const std::string& rpath, bool list_only,
-                  CompressionType compression, bool dry_run) {
+                  CompressionType compression, bool dry_run, bool quiet) {
     SyncConnection sc;
     if (!sc.IsValid()) return false;
+    sc.SetQuiet(quiet);
 
     bool success = copy_local_dir_remote(sc, lpath, rpath, true, list_only, compression, dry_run);
     if (!list_only) {
