@@ -23,9 +23,9 @@
 
 bool do_sync_ls(const char* path);
 bool do_sync_push(const std::vector<const char*>& srcs, const char* dst, bool sync,
-                  CompressionType compression, bool dry_run);
+                  CompressionType compression, bool dry_run, bool quiet);
 bool do_sync_pull(const std::vector<const char*>& srcs, const char* dst, bool copy_attrs,
-                  CompressionType compression, const char* name = nullptr);
+                  CompressionType compression, const char* name = nullptr, bool quiet = false);
 
 bool do_sync_sync(const std::string& lpath, const std::string& rpath, bool list_only,
-                  CompressionType compression, bool dry_run);
+                  CompressionType compression, bool dry_run, bool quiet);
