@@ -130,16 +130,14 @@ int adb_server_main(int is_daemon, const std::string& socket_spec, const char* o
 
     atexit(adb_server_cleanup);
 
-    init_transport_registration();
     init_reconnect_handler();
 
-    adb_wifi_init();
     if (!getenv("ADB_MDNS") || strcmp(getenv("ADB_MDNS"), "0") != 0) {
         init_mdns_transport_discovery();
     }
 
     if (!getenv("ADB_USB") || strcmp(getenv("ADB_USB"), "0") != 0) {
-        if (should_use_libusb()) {
+        if (is_libusb_enabled()) {
             libusb::usb_init();
         } else {
             usb_init();
@@ -159,7 +157,7 @@ int adb_server_main(int is_daemon, const std::string& socket_spec, const char* o
     // If we told a previous adb server to quit because of version mismatch, we can get to this
     // point before it's finished exiting. Retry for a while to give it some time. Don't actually
     // accept any connections until adb_wait_for_device_initialization finishes below.
-    while (install_listener(socket_spec, "*smartsocket*", nullptr, INSTALL_LISTENER_DISABLED,
+    while (install_listener(socket_spec, kSmartSocketConnectTo, nullptr, INSTALL_LISTENER_DISABLED,
                             nullptr, &error) != INSTALL_STATUS_OK) {
         if (std::chrono::steady_clock::now() - start > 0.5s) {
             LOG(FATAL) << "could not install *smartsocket* listener: " << error;
