@@ -19,6 +19,7 @@ from __future__ import print_function
 
 import contextlib
 import hashlib
+import io
 import os
 import posixpath
 import random
@@ -35,6 +36,8 @@ import threading
 import time
 import unittest
 
+import proto.adb_host_pb2 as adb_host_proto
+
 from datetime import datetime
 
 import adb
@@ -57,7 +60,8 @@ def requires_non_root(func):
 
 
 class DeviceTest(unittest.TestCase):
-    device = adb.get_device()
+    def setUp(self) -> None:
+        self.device = adb.get_device()
 
 
 class AbbTest(DeviceTest):
@@ -135,10 +139,10 @@ class ForwardReverseTest(DeviceTest):
         self.assertEqual('', msg.strip(),
                          'Forwarding list must be empty to run this test.')
 
-        s = socket.create_connection(("localhost", 5037))
-        service = b"host-serial:%s:forward:tcp:5566;tcp:6655" % serialno
-        cmd = b"%04x%s" % (len(service), service)
-        s.sendall(cmd)
+        with socket.create_connection(("localhost", 5037)) as s:
+            service = b"host-serial:%s:forward:tcp:5566;tcp:6655" % serialno
+            cmd = b"%04x%s" % (len(service), service)
+            s.sendall(cmd)
 
         msg = self.device.forward_list()
         self.assertTrue(re.search(r'tcp:5566.+tcp:6655', msg))
@@ -488,13 +492,13 @@ class ShellTest(DeviceTest):
 
         script = ";".join([x.strip() for x in script.strip().splitlines()])
 
-        process = self.device.shell_popen([script], kill_atexit=False,
-                                          stdin=subprocess.PIPE,
-                                          stdout=subprocess.PIPE)
+        with self.device.shell_popen([script], kill_atexit=False,
+                                     stdin=subprocess.PIPE,
+                                     stdout=subprocess.PIPE) as process:
 
-        self.assertEqual(b"Waiting\n", process.stdout.readline())
-        process.send_signal(signal.SIGINT)
-        process.wait()
+            self.assertEqual(b"Waiting\n", process.stdout.readline())
+            process.send_signal(signal.SIGINT)
+            process.wait()
 
         # Waiting for the local adb to finish is insufficient, since it hangs
         # up immediately.
@@ -676,15 +680,6 @@ def compute_md5(string):
     return hsh.hexdigest()
 
 
-def get_md5_prog(device):
-    """Older platforms (pre-L) had the name md5 rather than md5sum."""
-    try:
-        device.shell(['md5sum', '/proc/uptime'])
-        return 'md5sum'
-    except adb.ShellError:
-        return 'md5'
-
-
 class HostFile(object):
     def __init__(self, handle, checksum):
         self.handle = handle
@@ -732,7 +727,7 @@ def make_random_device_files(device, in_dir, num_files, prefix='device_tmpfile')
 
         device.shell(['dd', 'if=/dev/urandom', 'of={}'.format(full_path),
                       'bs={}'.format(size), 'count=1'])
-        dev_md5, _ = device.shell([get_md5_prog(device), full_path])[0].split()
+        dev_md5, _ = device.shell(['md5sum', full_path])[0].split()
 
         files.append(DeviceFile(dev_md5, full_path))
     return files
@@ -745,6 +740,7 @@ class FileOperationsTest:
         DEVICE_TEMP_DIR = SCRATCH_DIR + '/adb_test_dir'
 
         def setUp(self):
+            super().setUp()
             self.previous_env = os.environ.get("ADB_COMPRESSION")
             os.environ["ADB_COMPRESSION"] = self.compression
 
@@ -755,8 +751,7 @@ class FileOperationsTest:
                 os.environ["ADB_COMPRESSION"] = self.previous_env
 
         def _verify_remote(self, checksum, remote_path):
-            dev_md5, _ = self.device.shell([get_md5_prog(self.device),
-                                            remote_path])[0].split()
+            dev_md5, _ = self.device.shell(['md5sum', remote_path])[0].split()
             self.assertEqual(checksum, dev_md5)
 
         def _verify_local(self, checksum, local_path):
@@ -983,8 +978,7 @@ class FileOperationsTest:
                    'of={}'.format(self.DEVICE_TEMP_FILE), 'bs=1024',
                    'count={}'.format(kbytes)]
             self.device.shell(cmd)
-            dev_md5, _ = self.device.shell(
-                [get_md5_prog(self.device), self.DEVICE_TEMP_FILE])[0].split()
+            dev_md5, _ = self.device.shell(['md5sum', self.DEVICE_TEMP_FILE])[0].split()
             self._test_pull(self.DEVICE_TEMP_FILE, dev_md5)
             self.device.shell_nocheck(['rm', self.DEVICE_TEMP_FILE])
 
@@ -1195,8 +1189,7 @@ class FileOperationsTest:
             for temp_file in temp_files:
                 device_full_path = posixpath.join(
                     device_dir, temp_file.base_name)
-                dev_md5, _ = device.shell(
-                    [get_md5_prog(self.device), device_full_path])[0].split()
+                dev_md5, _ = device.shell(['md5sum', device_full_path])[0].split()
                 self.assertEqual(temp_file.checksum, dev_md5)
 
         def test_sync(self):
@@ -1280,7 +1273,7 @@ class FileOperationsTest:
 
 
         def test_push_dry_run_nonexistent_file(self):
-            """Push with dry run."""
+            """Push with dry run (non-existent file)."""
 
             for file_size in [8, 1024 * 1024]:
                 try:
@@ -1508,8 +1501,6 @@ class SocketTest(DeviceTest):
 
         Bug: http://b/74616284
         """
-        s = socket.create_connection(("localhost", 5037))
-
         def adb_length_prefixed(string):
             encoded = string.encode("utf8")
             result = b"%04x%s" % (len(encoded), encoded)
@@ -1520,36 +1511,38 @@ class SocketTest(DeviceTest):
         else:
             transport_string = "host:transport-any"
 
-        s.sendall(adb_length_prefixed(transport_string))
-        response = s.recv(4)
-        self.assertEqual(b"OKAY", response)
+        with socket.create_connection(("localhost", 5037)) as s:
 
-        shell_string = "shell:sleep 0.5; dd if=/dev/zero bs=1m count=1 status=none; echo foo"
-        s.sendall(adb_length_prefixed(shell_string))
+            s.sendall(adb_length_prefixed(transport_string))
+            response = s.recv(4)
+            self.assertEqual(b"OKAY", response)
 
-        response = s.recv(4)
-        self.assertEqual(b"OKAY", response)
+            shell_string = "shell:sleep 0.5; dd if=/dev/zero bs=1m count=1 status=none; echo foo"
+            s.sendall(adb_length_prefixed(shell_string))
 
-        # Spawn a thread that dumps garbage into the socket until failure.
-        def spam():
-            buf = b"\0" * 16384
-            try:
-                while True:
-                    s.sendall(buf)
-            except Exception as ex:
-                print(ex)
+            response = s.recv(4)
+            self.assertEqual(b"OKAY", response)
 
-        thread = threading.Thread(target=spam)
-        thread.start()
+            # Spawn a thread that dumps garbage into the socket until failure.
+            def spam():
+                buf = b"\0" * 16384
+                try:
+                    while True:
+                        s.sendall(buf)
+                except Exception as ex:
+                    print(ex)
 
-        time.sleep(1)
+            thread = threading.Thread(target=spam)
+            thread.start()
 
-        received = b""
-        while True:
-            read = s.recv(512)
-            if len(read) == 0:
-                break
-            received += read
+            time.sleep(1)
+
+            received = b""
+            while True:
+                read = s.recv(512)
+                if len(read) == 0:
+                    break
+                received += read
 
         self.assertEqual(1024 * 1024 + len("foo\n"), len(received))
         thread.join()
@@ -1761,15 +1754,144 @@ class WindowsConsoleTest(DeviceTest):
                 console_output = read_screen(screen)
                 self.assertEqual(unicode_string, console_output)
 
+class DevicesListing(DeviceTest):
 
-def main():
-    random.seed(0)
-    if len(adb.get_devices()) > 0:
-        suite = unittest.TestLoader().loadTestsFromName(__name__)
-        unittest.TextTestRunner(verbosity=3).run(suite)
-    else:
-        print('Test suite must be run with attached devices')
+    serial = subprocess.check_output(['adb', 'get-serialno']).strip().decode("utf-8")
+    # def get_serial(self):
+    #     return subprocess.check_output(self.device.adb_cmd + ['get-serialno']).strip().decode("utf-8")
 
+    def test_devices(self):
+        with subprocess.Popen(['adb', 'devices'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            lines = list(map(lambda b: b.decode("utf-8"), proc.stdout.readlines()))
+            self.assertEqual(len(lines), 3)
+            line = lines[1]
+            self.assertTrue(self.serial in line)
+            self.assertFalse("{" in line)
+            self.assertFalse("}" in line)
+            self.assertTrue("device" in line)
+            self.assertFalse("product" in line)
+            self.assertFalse("transport" in line)
+
+    def test_devices_l(self):
+        with subprocess.Popen(['adb', 'devices', '-l'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            lines = list(map(lambda b: b.decode("utf-8"), proc.stdout.readlines()))
+            self.assertEqual(len(lines), 3)
+            line = lines[1]
+            self.assertTrue(self.serial in line)
+            self.assertFalse("{" in line)
+            self.assertFalse("}" in line)
+            self.assertTrue("device" in line)
+            self.assertTrue("product" in line)
+            self.assertTrue("transport" in line)
+
+    def test_track_devices(self):
+        with subprocess.Popen(['adb', 'track-devices'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            with io.TextIOWrapper(proc.stdout, encoding='utf8') as reader:
+                output_size = int(reader.read(4), 16)
+                output = reader.read(output_size)
+                self.assertFalse("{" in output)
+                self.assertFalse("}" in output)
+                self.assertTrue(self.serial in output)
+                self.assertTrue("device" in output)
+                self.assertFalse("product" in output)
+                self.assertFalse("transport" in output)
+            proc.terminate()
+
+    def test_track_devices_l(self):
+        with subprocess.Popen(['adb', 'track-devices', '-l'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            with io.TextIOWrapper(proc.stdout, encoding='utf8') as reader:
+                output_size = int(reader.read(4), 16)
+                output = reader.read(output_size)
+                self.assertFalse("{" in output)
+                self.assertFalse("}" in output)
+                self.assertTrue(self.serial in output)
+                self.assertTrue("device" in output)
+                self.assertTrue("product" in output)
+                self.assertTrue("transport" in output)
+            proc.terminate()
+
+    def test_track_devices_proto_text(self):
+        with subprocess.Popen(['adb', 'track-devices', '--proto-text'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            with io.TextIOWrapper(proc.stdout, encoding='utf8') as reader:
+                output_size = int(reader.read(4), 16)
+                output = reader.read(output_size)
+                self.assertTrue("{" in output)
+                self.assertTrue("}" in output)
+                self.assertTrue(self.serial in output)
+                self.assertTrue("device" in output)
+                self.assertTrue("product" in output)
+                self.assertTrue("connection_type" in output)
+            proc.terminate()
+
+    def test_track_devices_proto_binary(self):
+        with subprocess.Popen(['adb', 'track-devices', '--proto-binary'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+
+            output_size = int(proc.stdout.read(4).decode("utf-8"), 16)
+            proto = proc.stdout.read(output_size)
+
+            devices = adb_host_proto.Devices()
+            devices.ParseFromString(proto)
+
+            device = devices.device[0]
+            self.assertTrue(device.serial == self.serial)
+            self.assertFalse(device.bus_address == "")
+            self.assertFalse(device.product == "")
+            self.assertFalse(device.model == "")
+            self.assertFalse(device.device == "")
+            self.assertTrue(device.negotiated_speed == int(device.negotiated_speed))
+            self.assertTrue(device.max_speed == int(device.max_speed))
+            self.assertTrue(device.transport_id == int(device.transport_id))
+
+            proc.terminate()
+
+class DevicesListing(DeviceTest):
+
+    serial = subprocess.check_output(['adb', 'get-serialno']).strip().decode("utf-8")
+
+    def test_track_app_appinfo(self):
+        return # Disabled until b/301491148 is fixed.
+        # (Exported FeatureFlags cannot be read-only)
+        subprocess.check_output(['adb', 'install', '-t', 'adb1.apk']).strip().decode("utf-8")
+        subprocess.check_output(['adb', 'install', '-t', 'adb2.apk']).strip().decode("utf-8")
+        subprocess.check_output(['adb', 'shell', 'am', 'start', '-W', 'adb.test.app1/.MainActivity']).strip().decode("utf-8")
+        subprocess.check_output(['adb', 'shell', 'am', 'start', '-W', 'adb.test.app2/.MainActivity']).strip().decode("utf-8")
+        subprocess.check_output(['adb', 'shell', 'am', 'start', '-W', 'adb.test.app1/.OwnProcessActivity']).strip().decode("utf-8")
+        with subprocess.Popen(['adb', 'track-app', '--proto-binary'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            output_size = int(proc.stdout.read(4).decode("utf-8"), 16)
+            proto = proc.stdout.read(output_size)
+
+            apps = proto_track_app.AppProcesses()
+            apps.ParseFromString(proto)
+
+            foundAdbAppDefProc = False
+            foundAdbAppOwnProc = False
+            for app in apps.process:
+                if (app.process_name == "adb.test.process.name"):
+                    foundAdbAppDefProc = True
+                    self.assertTrue(app.debuggable)
+                    self.assertTrue("adb.test.app1" in app.package_names)
+                    self.assertTrue("adb.test.app2" in app.package_names)
+
+                if (app.process_name == "adb.test.own.process"):
+                    foundAdbAppOwnProc = True
+                    self.assertTrue(app.debuggable)
+                    self.assertTrue("adb.test.app1" in app.package_names)
+
+            self.assertTrue(foundAdbAppDefProc)
+            self.assertTrue(foundAdbAppOwnProc)
+            proc.terminate()
+
+class ServerStatus(unittest.TestCase):
+    def test_server_status(self):
+        with subprocess.Popen(['adb', 'server-status'], stdin=subprocess.PIPE, stdout=subprocess.PIPE) as proc:
+            lines = list(map(lambda b: b.decode("utf-8"), proc.stdout.readlines()))
+            self.assertTrue("usb_backend" in lines[0])
+            self.assertTrue("mdns_backend" in lines[1])
+            self.assertTrue("version" in lines[2])
+            self.assertTrue("build" in lines[3])
+            self.assertTrue("executable_absolute_path" in lines[4])
+            self.assertTrue("log_absolute_path" in lines[5])
 
 if __name__ == '__main__':
-    main()
+    random.seed(0)
+    unittest.main()
