@@ -16,27 +16,19 @@
 
 #pragma once
 
+#include <elf.h>
 #include <stddef.h>
 
 #include <memory>
 #include <mutex>
 #include <string>
 #include <unordered_map>
-#include <utility>
 
 #include <unwindstack/Arch.h>
 #include <unwindstack/ElfInterface.h>
 #include <unwindstack/Memory.h>
 #include <unwindstack/SharedString.h>
 
-#if !defined(EM_AARCH64)
-#define EM_AARCH64 183
-#endif
-
-#if !defined(EM_RISCV)
-#define EM_RISCV 243
-#endif
-
 namespace unwindstack {
 
 // Forward declaration.
@@ -45,7 +37,7 @@ class Regs;
 
 class Elf {
  public:
-  Elf(Memory* memory) : memory_(memory) {}
+  Elf(std::shared_ptr<Memory>& memory) : memory_(memory) {}
   virtual ~Elf() = default;
 
   bool Init();
@@ -67,7 +59,7 @@ class Elf {
   bool Step(uint64_t rel_pc, Regs* regs, Memory* process_memory, bool* finished,
             bool* is_signal_frame);
 
-  ElfInterface* CreateInterfaceFromMemory(Memory* memory);
+  ElfInterface* CreateInterfaceFromMemory(std::shared_ptr<Memory>& memory);
 
   std::string GetBuildID();
 
@@ -91,7 +83,7 @@ class Elf {
 
   ArchEnum arch() { return arch_; }
 
-  Memory* memory() { return memory_.get(); }
+  std::shared_ptr<Memory> memory() { return memory_; }
 
   ElfInterface* interface() { return interface_.get(); }
 
@@ -124,14 +116,13 @@ class Elf {
   bool valid_ = false;
   int64_t load_bias_ = 0;
   std::unique_ptr<ElfInterface> interface_;
-  std::unique_ptr<Memory> memory_;
+  std::shared_ptr<Memory> memory_;
   uint32_t machine_type_;
   uint8_t class_type_;
   ArchEnum arch_;
   // Protect calls that can modify internal state of the interface object.
   std::mutex lock_;
 
-  std::unique_ptr<Memory> gnu_debugdata_memory_;
   std::unique_ptr<ElfInterface> gnu_debugdata_interface_;
 
   static bool cache_enabled_;
