@@ -44,9 +44,14 @@ enum TextFormatIR {
 
 enum CompatibilityStatusIR {
   Compatible = 0,
+  // Changing classes or enums not referenced by functions or variables.
   UnreferencedChanges = 1,
+  // Adding symbols.
+  ElfExtension = 2,
+  // Adding functions, classes, class members, etc.
   Extension = 4,
   Incompatible = 8,
+  // Removing symbols.
   ElfIncompatible = 16
 };
 
@@ -65,16 +70,12 @@ static inline CompatibilityStatusIR operator&(CompatibilityStatusIR f,
 }
 
 enum AccessSpecifierIR {
+  // Ordered from the least to the most restricted.
   PublicAccess = 1,
   ProtectedAccess = 2,
   PrivateAccess = 3
 };
 
-static inline bool IsAccessDowngraded(AccessSpecifierIR old_access,
-                                      AccessSpecifierIR new_access) {
-  return old_access < new_access;
-}
-
 enum LinkableMessageKind {
   RecordTypeKind,
   EnumTypeKind,
@@ -334,9 +335,14 @@ class TemplatedArtifactIR {
 class RecordFieldIR : public ReferencesOtherType {
  public:
   RecordFieldIR(const std::string &name, const std::string &type,
-                uint64_t offset, AccessSpecifierIR access)
-      : ReferencesOtherType(type), name_(name), offset_(offset),
-        access_(access) {}
+                uint64_t offset, AccessSpecifierIR access, bool is_bit_field,
+                uint64_t bit_width)
+      : ReferencesOtherType(type),
+        name_(name),
+        offset_(offset),
+        access_(access),
+        is_bit_field_(is_bit_field),
+        bit_width_(bit_width) {}
 
   RecordFieldIR() {}
 
@@ -352,10 +358,16 @@ class RecordFieldIR : public ReferencesOtherType {
     return access_;
   }
 
+  bool IsBitField() const { return is_bit_field_; }
+
+  uint64_t GetBitWidth() const { return bit_width_; }
+
  protected:
   std::string name_;
   uint64_t offset_ = 0;
   AccessSpecifierIR access_ = AccessSpecifierIR::PublicAccess;
+  bool is_bit_field_ = false;
+  uint64_t bit_width_ = 0;
 };
 
 class RecordTypeIR : public TypeIR, public TemplatedArtifactIR {
@@ -801,7 +813,7 @@ class TypeDefinition {
 
 class ModuleIR {
  public:
-  ModuleIR(const std::set<std::string> *exported_headers)
+  ModuleIR(const std::set<std::string> *exported_headers = nullptr)
       : exported_headers_(exported_headers) {}
 
   const std::string &GetCompilationUnitPath() const {
