@@ -16,7 +16,6 @@
 #define HEADER_CHECKER_REPR_JSON_API_H_
 
 #include <memory>
-#include <set>
 #include <string>
 
 
@@ -26,12 +25,12 @@ namespace repr {
 
 class IRDumper;
 class IRReader;
-
+class ModuleIR;
 
 std::unique_ptr<IRDumper> CreateJsonIRDumper(const std::string &dump_path);
 
 std::unique_ptr<IRReader> CreateJsonIRReader(
-    const std::set<std::string> *exported_headers);
+    std::unique_ptr<ModuleIR> module_ir);
 
 
 }  // namespace repr
