@@ -16,7 +16,6 @@
 #define HEADER_CHECKER_REPR_PROTOBUF_API_H_
 
 #include <memory>
-#include <set>
 #include <string>
 
 
@@ -27,12 +26,12 @@ namespace repr {
 class IRDiffDumper;
 class IRDumper;
 class IRReader;
-
+class ModuleIR;
 
 std::unique_ptr<IRDumper> CreateProtobufIRDumper(const std::string &dump_path);
 
 std::unique_ptr<IRReader> CreateProtobufIRReader(
-    const std::set<std::string> *exported_headers);
+    std::unique_ptr<ModuleIR> module_ir);
 
 std::unique_ptr<IRDiffDumper> CreateProtobufIRDiffDumper(
     const std::string &dump_path);
