@@ -174,8 +174,9 @@ TEST(VersionScriptParserTest, ParseSymbolTagsIntroduced) {
     ASSERT_TRUE(result);
 
     const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
-
-    EXPECT_TRUE(funcs.empty());
+    // This may be an undefined behavior. ndkstubgen includes it in llndk mode,
+    // but excludes it in ndk mode.
+    EXPECT_THAT(funcs, ElementsAre(Key("test5")));
   }
 
   {
@@ -355,6 +356,142 @@ TEST(VersionScriptParserTest, ExcludeSymbolTags) {
 }
 
 
+TEST(VersionScriptParserTest, IncludeSymbolTags) {
+  static const char testdata[] = R"TESTDATA(
+    LIBEX_1.0 {
+      global:
+        always;  # unknown
+        api34;  # introduced=34
+        api35;  # introduced=35
+        llndk202404;  # llndk=202404
+        llndk202504;  # llndk=202504
+        systemapi;  # systemapi
+        systemapi_llndk;  # systemapi llndk
+    };
+  )TESTDATA";
+
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(34);
+    parser.AddModeTag("llndk=202404");
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_THAT(funcs, ElementsAre(Key("always"), Key("api34"),
+                                   Key("llndk202404"), Key("systemapi_llndk")));
+  }
+
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(34);
+    parser.AddModeTag("llndk");
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_THAT(funcs,
+                ElementsAre(Key("always"), Key("api34"), Key("llndk202404"),
+                            Key("llndk202504"), Key("systemapi_llndk")));
+  }
+
+  // Include all mode tags
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(utils::FUTURE_API_LEVEL);
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_THAT(funcs, ElementsAre(Key("always"), Key("api34"), Key("api35"),
+                                   Key("llndk202404"), Key("llndk202504"),
+                                   Key("systemapi"), Key("systemapi_llndk")));
+  }
+
+  // Exclude all mode tags
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(utils::FUTURE_API_LEVEL);
+    parser.AddModeTag("none");
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_THAT(funcs, ElementsAre(Key("always"), Key("api34"), Key("api35")));
+  }
+}
+
+
+TEST(VersionScriptParserTest, SetModeTagPolicy) {
+  static const char testdata[] = R"TESTDATA(
+    LIBEX_1.0 {  # introduced=36
+        api36;
+        api36_llndk202504;
+        api36_llndk202504;  # llndk=202504
+        llndk202504;  # llndk=202504
+    };
+  )TESTDATA";
+
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(35);
+    parser.AddModeTag("llndk=202504");
+    parser.SetModeTagPolicy(ModeTagPolicy::MatchTagAndApi);
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_TRUE(funcs.empty());
+  }
+
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(35);
+    parser.AddModeTag("llndk=202504");
+    parser.SetModeTagPolicy(ModeTagPolicy::MatchTagOnly);
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_THAT(funcs,
+                ElementsAre(Key("api36_llndk202504"), Key("llndk202504")));
+  }
+
+  {
+    VersionScriptParser parser;
+    parser.SetApiLevel(36);
+    parser.SetModeTagPolicy(ModeTagPolicy::MatchTagAndApi);
+
+    std::istringstream stream(testdata);
+    std::unique_ptr<ExportedSymbolSet> result(parser.Parse(stream));
+    ASSERT_TRUE(result);
+
+    const ExportedSymbolSet::FunctionMap &funcs = result->GetFunctions();
+
+    EXPECT_THAT(funcs, ElementsAre(Key("api36"), Key("api36_llndk202504"),
+                                   Key("llndk202504")));
+  }
+}
+
+
 TEST(VersionScriptParserTest, ParseExternCpp) {
   static const char testdata[] = R"TESTDATA(
     LIBEX_1.0 {
