@@ -15,8 +15,7 @@
 #ifndef STRING_UTILS_H_
 #define STRING_UTILS_H_
 
-#include <llvm/ADT/Optional.h>
-
+#include <optional>
 #include <set>
 #include <string>
 #include <vector>
@@ -38,7 +37,7 @@ bool EndsWith(std::string_view s, std::string_view suffix);
 std::vector<std::string_view> Split(std::string_view s,
                                     std::string_view delim_chars);
 
-llvm::Optional<int> ParseInt(const std::string &s);
+std::optional<int> ParseInt(const std::string &s);
 
 bool ParseBool(const std::string &s);
 
